package com.AccountName.thumbnailmaker.scale;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.ActivityManager.MemoryInfo;
import android.content.ContentResolver;
import android.content.Context;
import android.content.res.AssetFileDescriptor;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory.Options;
import android.graphics.BitmapRegionDecoder;
import android.graphics.Point;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Build.VERSION;
import android.support.v4.media.session.PlaybackStateCompat;
import android.text.TextUtils;
import android.util.Log;
import java.io.File;
import java.io.FileFilter;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Semaphore;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.regex.Pattern;

public class SkiaPooledImageRegionDecoder implements ImageRegionDecoder {
    private static final String ASSET_PREFIX = "file:///android_asset/";
    private static final String FILE_PREFIX = "file://";
    private static final String RESOURCE_PREFIX = "android.resource://";
    private static final String TAG = "SkiaPooledImageRegionDecoder";
    private static boolean debug = false;
    private final Config bitmapConfig;
    private Context context;
    private final ReadWriteLock decoderLock;
    private DecoderPool decoderPool;
    private long fileLength;
    private Point imageDimensions;
    private final AtomicBoolean lazyInited;
    private Uri uri;

    private static class DecoderPool {
        private final Semaphore available;
        private Map<BitmapRegionDecoder, Boolean> decoders;

        private DecoderPool() {
            this.available = new Semaphore(0, true);
            this.decoders = new ConcurrentHashMap();
        }



        private synchronized boolean isEmpty() {
            return this.decoders.isEmpty();
        }

        private synchronized int size() {
            return this.decoders.size();
        }

        private BitmapRegionDecoder acquire() {
            this.available.acquireUninterruptibly();
            return getNextAvailable();
        }

        private void release(BitmapRegionDecoder bitmapRegionDecoder) {
            if (markAsUnused(bitmapRegionDecoder)) {
                this.available.release();
            }
        }

        private synchronized void add(BitmapRegionDecoder bitmapRegionDecoder) {
            this.decoders.put(bitmapRegionDecoder, Boolean.valueOf(false));
            this.available.release();
        }

        private synchronized void recycle() {
            while (!this.decoders.isEmpty()) {
                BitmapRegionDecoder acquire = acquire();
                acquire.recycle();
                this.decoders.remove(acquire);
            }
        }

        private synchronized BitmapRegionDecoder getNextAvailable() {
            for (Entry entry : this.decoders.entrySet()) {
                if (!((Boolean) entry.getValue()).booleanValue()) {
                    entry.setValue(Boolean.valueOf(true));
                    return (BitmapRegionDecoder) entry.getKey();
                }
            }
            return null;
        }

        private synchronized boolean markAsUnused(BitmapRegionDecoder bitmapRegionDecoder) {
            for (Entry entry : this.decoders.entrySet()) {
                if (bitmapRegionDecoder == entry.getKey()) {
                    if (!((Boolean) entry.getValue()).booleanValue()) {
                        return false;
                    }
                    entry.setValue(Boolean.valueOf(false));
                    return true;
                }
            }
            return false;
        }
    }

    public SkiaPooledImageRegionDecoder() {
        this(null);
    }

    public SkiaPooledImageRegionDecoder(Config config) {
        this.decoderPool = new DecoderPool();
        this.decoderLock = new ReentrantReadWriteLock(true);
        this.fileLength = Long.MAX_VALUE;
        this.imageDimensions = new Point(0, 0);
        this.lazyInited = new AtomicBoolean(false);
        Config preferredBitmapConfig = SubsamplingScaleImageView.getPreferredBitmapConfig();
        if (config != null) {
            this.bitmapConfig = config;
        } else if (preferredBitmapConfig != null) {
            this.bitmapConfig = preferredBitmapConfig;
        } else {
            this.bitmapConfig = Config.RGB_565;
        }
    }

    public static void setDebug(boolean z) {
        debug = z;
    }

    public Point init(Context context, Uri uri) throws Exception {
        this.context = context;
        this.uri = uri;
        initialiseDecoder();
        return this.imageDimensions;
    }

    private void lazyInit() {
        if (this.lazyInited.compareAndSet(false, true) && this.fileLength < Long.MAX_VALUE) {
            debug("Starting lazy init of additional decoders");
            new Thread() {
                public void run() {
                    while (SkiaPooledImageRegionDecoder.this.decoderPool != null && SkiaPooledImageRegionDecoder.this.allowAdditionalDecoder(SkiaPooledImageRegionDecoder.this.decoderPool.size(), SkiaPooledImageRegionDecoder.this.fileLength)) {
                        try {
                            if (SkiaPooledImageRegionDecoder.this.decoderPool != null) {
                                long currentTimeMillis = System.currentTimeMillis();
                                SkiaPooledImageRegionDecoder.this.debug("Starting decoder");
                                SkiaPooledImageRegionDecoder.this.initialiseDecoder();
                                long currentTimeMillis2 = System.currentTimeMillis();
                                SkiaPooledImageRegionDecoder skiaPooledImageRegionDecoder = SkiaPooledImageRegionDecoder.this;
                                StringBuilder stringBuilder = new StringBuilder();
                                stringBuilder.append("Started decoder, took ");
                                stringBuilder.append(currentTimeMillis2 - currentTimeMillis);
                                stringBuilder.append("ms");
                                skiaPooledImageRegionDecoder.debug(stringBuilder.toString());
                            }
                        } catch (Exception e) {
                            SkiaPooledImageRegionDecoder skiaPooledImageRegionDecoder2 = SkiaPooledImageRegionDecoder.this;
                            StringBuilder stringBuilder2 = new StringBuilder();
                            stringBuilder2.append("Failed to start decoder: ");
                            stringBuilder2.append(e.getMessage());
                            skiaPooledImageRegionDecoder2.debug(stringBuilder2.toString());
                        }
                    }
                }
            }.start();
        }
    }

    /* JADX WARNING: Removed duplicated region for block: B:70:0x0153 A:{SYNTHETIC, Splitter:B:70:0x0153} */
    /* Code decompiled incorrectly, please refer to instructions dump. */
    private void initialiseDecoder() throws Exception {
        BitmapRegionDecoder newInstance = null;
        Throwable th;
        String uri = this.uri.toString();
        long j = Long.MAX_VALUE;
        if (uri.startsWith(RESOURCE_PREFIX)) {
            Resources resources;
            int identifier;
            uri = this.uri.getAuthority();
            if (this.context.getPackageName().equals(uri)) {
                resources = this.context.getResources();
            } else {
                resources = this.context.getPackageManager().getResourcesForApplication(uri);
            }
            List pathSegments = this.uri.getPathSegments();
            int size = pathSegments.size();
            if (size == 2 && ((String) pathSegments.get(0)).equals("drawable")) {
                identifier = resources.getIdentifier((String) pathSegments.get(1), "drawable", uri);
            } else {
                if (size == 1 && TextUtils.isDigitsOnly((CharSequence) pathSegments.get(0))) {
                    try {
                        identifier = Integer.parseInt((String) pathSegments.get(0));
                    } catch (NumberFormatException unused) {
                    }
                }
                identifier = 0;
            }
            try {
                j = this.context.getResources().openRawResourceFd(identifier).getLength();
            } catch (Exception unused2) {
            }
            newInstance = BitmapRegionDecoder.newInstance(this.context.getResources().openRawResource(identifier), false);
        } else if (uri.startsWith(ASSET_PREFIX)) {
            uri = uri.substring(ASSET_PREFIX.length());
            try {
                j = this.context.getAssets().openFd(uri).getLength();
            } catch (Exception unused3) {
            }
            newInstance = BitmapRegionDecoder.newInstance(this.context.getAssets().open(uri, 1), false);
        } else if (uri.startsWith(FILE_PREFIX)) {
            BitmapRegionDecoder newInstance2 = BitmapRegionDecoder.newInstance(uri.substring(FILE_PREFIX.length()), false);
            try {
                File file = new File(uri);
                if (file.exists()) {
                    j = file.length();
                }
            } catch (Exception unused4) {
            }
            newInstance = newInstance2;
        } else {
            InputStream openInputStream;
            try {
                ContentResolver contentResolver = this.context.getContentResolver();
                openInputStream = contentResolver.openInputStream(this.uri);
                try {
                    newInstance = BitmapRegionDecoder.newInstance(openInputStream, false);
                    try {
                        AssetFileDescriptor openAssetFileDescriptor = contentResolver.openAssetFileDescriptor(this.uri, "r");
                        if (openAssetFileDescriptor != null) {
                            j = openAssetFileDescriptor.getLength();
                        }
                    } catch (Exception unused5) {
                    }
                    if (openInputStream != null) {
                        try {
                            openInputStream.close();
                        } catch (Exception unused6) {
                        }
                    }
                } catch (Throwable th2) {
                    th = th2;
                    if (openInputStream != null) {
                        try {
                            openInputStream.close();
                        } catch (Exception unused7) {
                        }
                    }
                    throw th;
                }
            } catch (Throwable th3) {
                openInputStream = null;
                th = th3;
                if (openInputStream != null) {
                }
                try {
                    throw th;
                } catch (Throwable throwable) {
                    throwable.printStackTrace();
                }
            }
        }
        this.fileLength = j;
        this.imageDimensions.set(newInstance.getWidth(), newInstance.getHeight());
        this.decoderLock.writeLock().lock();
        try {
            if (this.decoderPool != null) {
                this.decoderPool.add(newInstance);
            }
            this.decoderLock.writeLock().unlock();
        } catch (Throwable th4) {
            this.decoderLock.writeLock().unlock();
        }
    }

    public Bitmap decodeRegion(Rect rect, int i) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Decode region ");
        stringBuilder.append(rect);
        stringBuilder.append(" on thread ");
        stringBuilder.append(Thread.currentThread().getName());
        debug(stringBuilder.toString());
        if (rect.width() < this.imageDimensions.x || rect.height() < this.imageDimensions.y) {
            lazyInit();
        }
        this.decoderLock.readLock().lock();
        BitmapRegionDecoder access$700;
        try {
            if (this.decoderPool != null) {
                access$700 = this.decoderPool.acquire();
                if (access$700 != null) {
                    if (!access$700.isRecycled()) {
                        Options options = new Options();
                        options.inSampleSize = i;
                        options.inPreferredConfig = this.bitmapConfig;
                        Bitmap decodeRegion = access$700.decodeRegion(rect, options);
                        if (decodeRegion == null) {
                            throw new RuntimeException("Skia image decoder returned null bitmap - image format may not be supported");
                        }
                        if (access$700 != null) {
                            this.decoderPool.release(access$700);
                        }
                        this.decoderLock.readLock().unlock();
                        return decodeRegion;
                    }
                }
                if (access$700 != null) {
                    this.decoderPool.release(access$700);
                }
            }
            throw new IllegalStateException("Cannot decode region after decoder has been recycled");
        } catch (Throwable th) {
            this.decoderLock.readLock().unlock();
        }
        return null;
    }

    public synchronized boolean isReady() {
        boolean z;
        z = (this.decoderPool == null || this.decoderPool.isEmpty()) ? false : true;
        return z;
    }

    public synchronized void recycle() {
        this.decoderLock.writeLock().lock();
        try {
            if (this.decoderPool != null) {
                this.decoderPool.recycle();
                this.decoderPool = null;
                this.context = null;
                this.uri = null;
            }
        } finally {
            this.decoderLock.writeLock().unlock();
        }
    }


    public boolean allowAdditionalDecoder(int i, long j) {
        if (i >= 4) {
            debug("No additional decoders allowed, reached hard limit (4)");
            return false;
        }
        long j2 = ((long) i) * j;
        if (j2 > 20971520) {
            debug("No additional encoders allowed, reached hard memory limit (20Mb)");
            return false;
        } else if (i >= getNumberOfCores()) {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("No additional encoders allowed, limited by CPU cores (");
            stringBuilder.append(getNumberOfCores());
            stringBuilder.append(")");
            debug(stringBuilder.toString());
            return false;
        } else if (isLowMemory()) {
            debug("No additional encoders allowed, memory is low");
            return false;
        } else {
            StringBuilder stringBuilder2 = new StringBuilder();
            stringBuilder2.append("Additional decoder allowed, current count is ");
            stringBuilder2.append(i);
            stringBuilder2.append(", estimated native memory ");
            stringBuilder2.append(j2 / PlaybackStateCompat.ACTION_SET_CAPTIONING_ENABLED);
            stringBuilder2.append("Mb");
            debug(stringBuilder2.toString());
            return true;
        }
    }

    private int getNumberOfCores() {
        if (VERSION.SDK_INT >= 17) {
            return Runtime.getRuntime().availableProcessors();
        }
        return getNumCoresOldPhones();
    }

    private int getNumCoresOldPhones() {
        try {
            return new File("/sys/devices/system/cpu/").listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return Pattern.matches("cpu[0-9]+", file.getName());
                }
            }).length;
        } catch (Exception unused) {
            return 1;
        }
    }

    private boolean isLowMemory() {
        ActivityManager activityManager = (ActivityManager) this.context.getSystemService(Context.ACTIVITY_SERVICE);
        if (activityManager == null) {
            return true;
        }
        MemoryInfo memoryInfo = new MemoryInfo();
        activityManager.getMemoryInfo(memoryInfo);
        return memoryInfo.lowMemory;
    }

    @SuppressLint("LongLogTag")
    private void debug(String str) {
        if (debug) {
            Log.d(TAG, str);
        }
    }
}
